#include "stdafx.h"
#include "d3dclass.h"
#include "inputclass.h"
#include "shadermanagerclass.h"
#include "texturemanagerclass.h"
#include "timerclass.h"
#include "userinterfaceclass.h"
#include "cameraclass.h"
#include "lightclass.h"
#include "positionclass.h"
#include "skydomeclass.h"
#include "terrainclass.h"
#include "zoneclass.h"


ZoneClass::ZoneClass()
{
	m_UserInterface = 0;
	m_Camera = 0;
	m_Light = 0;
	m_Position = 0;
	m_SkyDome = 0;
	m_Terrain = 0;
}


ZoneClass::ZoneClass(const ZoneClass& other)
{
}


ZoneClass::~ZoneClass()
{
}


bool ZoneClass::Initialize(D3DClass* Direct3D, HWND hwnd, int screenWidth, int screenHeight, float screenDepth)
{
	//  ̽ ü ϴ.
	m_UserInterface = new UserInterfaceClass;
	if(!m_UserInterface)
	{
		return false;
	}

	//  ̽ ü ʱȭմϴ.
	if(!m_UserInterface->Initialize(Direct3D, screenHeight, screenWidth))
	{
		MessageBox(hwnd, L"Could not initialize the user interface object.", L"Error", MB_OK);
		return false;
	}

	// ī޶ ü ϴ.
	m_Camera = new CameraClass;
	if(!m_Camera)
	{
		return false;
	}

	// ī޶ ʱ ġ ϰ  ʿ  ϴ.
	m_Camera->SetPosition(XMFLOAT3(0.0f, 0.0f, -10.0f));
	m_Camera->Render();
	m_Camera->RenderBaseViewMatrix();

	//  ü ϴ.
	m_Light = new LightClass;
	if(!m_Light)
	{
		return false;
	}

	//  ü ʱȭ մϴ.
	m_Light->SetDiffuseColor(1.0f, 1.0f, 1.0f, 1.0f);
	m_Light->SetDirection(-0.5f, -1.0f, -0.5f);

	// ġ ü ϴ.
	m_Position = new PositionClass;
	if(!m_Position)
	{
		return false;
	}

	// ʱ ġ ȸ մϴ.
	m_Position->SetPosition(XMFLOAT3(512.0f, 30.0f, -10.0f));
	m_Position->SetRotation(XMFLOAT3(0.0f, 0.0f, 0.0f));

	// ϴ  ü ϴ.
	m_SkyDome = new SkyDomeClass;
	if(!m_SkyDome)
	{
		return false;
	}

	// ϴ  ü ʱȭմϴ.
	if(!m_SkyDome->Initialize(Direct3D->GetDevice()))
	{
		MessageBox(hwnd, L"Could not initialize the sky dome object.", L"Error", MB_OK);
		return false;
	}

	//  ü ϴ.
	m_Terrain = new TerrainClass;
	if(!m_Terrain)
	{
		return false;
	}

	//  ü ʱȭմϴ.
	if(!m_Terrain->Initialize(Direct3D->GetDevice(), "../Dx11Terrain_20/data/setup.txt"))
	{
		MessageBox(hwnd, L"Could not initialize the terrain object.", L"Error", MB_OK);
		return false;
	}
	
	// ⺻ ǥ  UI մϴ.
	m_displayUI = true;

	// ̾   ó Ȱȭ մϴ.
	m_wireFrame = false;

	return true;
}


void ZoneClass::Shutdown()
{
	//  ü մϴ.
	if(m_Terrain)
	{
		m_Terrain->Shutdown();
		delete m_Terrain;
		m_Terrain = 0;
	}

	// ϴ  ü մϴ.
	if(m_SkyDome)
	{
		m_SkyDome->Shutdown();
		delete m_SkyDome;
		m_SkyDome = 0;
	}

	// ġ ü մϴ.
	if(m_Position)
	{
		delete m_Position;
		m_Position = 0;
	}

	//  ü մϴ.
	if(m_Light)
	{
		delete m_Light;
		m_Light = 0;
	}

	// ī޶ ü մϴ.
	if(m_Camera)
	{
		delete m_Camera;
		m_Camera = 0;
	}

	//  ̽ ü մϴ.
	if(m_UserInterface)
	{
		m_UserInterface->Shutdown();
		delete m_UserInterface;
		m_UserInterface = 0;
	}
}


bool ZoneClass::Frame(D3DClass* Direct3D, InputClass* Input, ShaderManagerClass* ShaderManager, TextureManagerClass* TextureManager, 
					  float frameTime, int fps)
{
	XMFLOAT3 pos, rot;

	//  Է ó մϴ.
	HandleMovementInput(Input, frameTime);

	//  Ʈ ġ / ȸ ɴϴ.
	m_Position->GetPosition(pos);
	m_Position->GetRotation(rot);

	//  ̽   ó մϴ.
	if(!m_UserInterface->Frame(Direct3D->GetDeviceContext(), fps, pos, rot))
	{
		return false;
	}

	// ׷ մϴ.
	if(!Render(Direct3D, ShaderManager, TextureManager))
	{
		return false;
	}

	return true;
}


void ZoneClass::HandleMovementInput(InputClass* Input, float frameTime)
{
	XMFLOAT3 pos, rot;

	// Ʈ  ġ ϱ  ð մϴ.
	m_Position->SetFrameTime(frameTime);

	// Է óմϴ.
	m_Position->TurnLeft(Input->IsLeftPressed());
	m_Position->TurnRight(Input->IsRightPressed());
	m_Position->MoveForward(Input->IsUpPressed());
	m_Position->MoveBackward(Input->IsDownPressed());
	m_Position->MoveUpward(Input->IsAPressed());
	m_Position->MoveDownward(Input->IsZPressed());
	m_Position->LookUpward(Input->IsPgUpPressed());
	m_Position->LookDownward(Input->IsPgDownPressed());

	//  Ʈ ġ / ȸ ɴϴ.
	m_Position->GetPosition(pos);
	m_Position->GetRotation(rot);

	// ī޶ ġ մϴ.
	m_Camera->SetPosition(pos);
	m_Camera->SetRotation(rot);

	//  ̽ ǥ θ մϴ.
	if(Input->IsF1Toggled())
	{
		m_displayUI = !m_displayUI;
	}

	//  ̾   θ մϴ.
	if(Input->IsF2Toggled())
	{
		m_wireFrame = !m_wireFrame;
	}
}


bool ZoneClass::Render(D3DClass* Direct3D, ShaderManagerClass* ShaderManager, TextureManagerClass* TextureManager)
{
	XMMATRIX worldMatrix, viewMatrix, projectionMatrix, baseViewMatrix, orthoMatrix;
	
	// ī޶ ġ    մϴ.
	m_Camera->Render();

	// ī޶  d3d ü ,     ɴϴ.
	Direct3D->GetWorldMatrix(worldMatrix);
	m_Camera->GetViewMatrix(viewMatrix);
	Direct3D->GetProjectionMatrix(projectionMatrix);
	m_Camera->GetBaseViewMatrix(baseViewMatrix);
	Direct3D->GetOrthoMatrix(orthoMatrix);
	
	// ī޶ ġ ´.
	XMFLOAT3 cameraPosition = m_Camera->GetPosition();

	//   ۸ .
	Direct3D->BeginScene(0.0f, 0.0f, 0.0f, 1.0f);

	// ޸ ø  Z ۸ ϴ.
	Direct3D->TurnOffCulling();
	Direct3D->TurnZBufferOff();

	// ī  ī޶ ġ ߽ ȯմϴ.
	worldMatrix = XMMatrixTranslation(cameraPosition.x, cameraPosition.y, cameraPosition.z);

	// ϴ  ̴ Ͽ ϴ  մϴ.
	m_SkyDome->Render(Direct3D->GetDeviceContext());
	
	if(!ShaderManager->RenderSkyDomeShader(Direct3D->GetDeviceContext(), m_SkyDome->GetIndexCount(), worldMatrix, viewMatrix,
										   projectionMatrix, m_SkyDome->GetApexColor(), m_SkyDome->GetCenterColor()))
	{
		return false;
	}

	//   缳մϴ.
	Direct3D->GetWorldMatrix(worldMatrix);

	// Z ۸ ʰ ޸ ø (culling)Ѵ.
	Direct3D->TurnZBufferOn();
	Direct3D->TurnOnCulling();
	
	// ʿ   ̾   մϴ.
	if(m_wireFrame)
	{
		Direct3D->EnableWireframe();
	}

	//  ̴ Ͽ  մϴ.
	m_Terrain->Render(Direct3D->GetDeviceContext());
	
	if(!ShaderManager->RenderTerrainShader(Direct3D->GetDeviceContext(), m_Terrain->GetIndexCount(), worldMatrix, viewMatrix,
										   projectionMatrix, TextureManager->GetTexture(0), TextureManager->GetTexture(1),
										   m_Light->GetDirection(), m_Light->GetDiffuseColor()))
	{
		return false;
	}

	//      ̾   ϴ.
	if(m_wireFrame)
	{
		Direct3D->DisableWireframe();  
	}

	//  ̽ մϴ.
	if(m_displayUI)
	{
		m_UserInterface->Render(Direct3D, ShaderManager, worldMatrix, baseViewMatrix, orthoMatrix);
	}

	//    ȭ鿡 ǥմϴ.
	Direct3D->EndScene();

	return true;
}