////////////////////////////////////////////////////////////////////////////////
// Filename: terrain_ps.hlsl
////////////////////////////////////////////////////////////////////////////////


//////////////
// TEXTURES //
//////////////
Texture2D shaderTexture : register(t0);
Texture2D normalTexture : register(t1);


//////////////
// SAMPLERS //
//////////////
SamplerState SampleType : register(s0);


//////////////////////
// CONSTANT BUFFERS //
//////////////////////
cbuffer LightBuffer
{
	float4 diffuseColor;
	float3 lightDirection;
	float padding;
};


//////////////
// TYPEDEFS //
//////////////
struct PixelInputType
{
	float4 position : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float3 tangent : TANGENT;
	float3 binormal : BINORMAL;
	float4 color : COLOR;
};


////////////////////////////////////////////////////////////////////////////////
// Pixel Shader
////////////////////////////////////////////////////////////////////////////////
float4 TerrainPixelShader(PixelInputType input) : SV_TARGET
{
	float4 textureColor;
	float3 lightDir;
	float4 bumpMap;
	float3 bumpNormal;
	float lightIntensity;
	float4 color;
	
	
	// ؽó ǥ ġ ÷ Ͽ ؽó ȼ  øմϴ.
	textureColor = shaderTexture.Sample(SampleType, input.tex);
	
	//    ؽó  մϴ.
	textureColor = saturate(textureColor * input.color * 2.0f);
	
	//     ŵϴ.
	lightDir = -lightDirection;
	
	//   Ͽ  ȼ   մϴ.
	bumpMap = normalTexture.Sample(SampleType, input.tex);
	bumpMap = (bumpMap * 2.0f) - 1.0f;
	bumpNormal = (bumpMap.x * input.tangent) + (bumpMap.y * input.binormal) + (bumpMap.z * input.normal);
	bumpNormal = normalize(bumpNormal);
	lightIntensity = saturate(dot(bumpNormal, lightDir));
	
	//     Ȯ       մϴ.
	color = saturate(diffuseColor * lightIntensity);
	
	// ؽó ȼ  Ȯ  Ͽ  ȼ   ϴ.
	color = color * textureColor;
	
	return color;
}